/* This file act as a bridge between CCEXtractor code and Bento 4 

An implementation for a stream reader that uses CCExtractor's buffer is needed. For now we're bypassing 
CCExtractor file stuff, therefore missing some of the things it brings */

#ifdef WITHBENTO4
/*----------------------------------------------------------------------
|   includes
+---------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>

#include "bento4/Ap4.h"
#include "bento4/Ap4AvcParser.h"
#include "bento4/Ap4BitStream.h"

#include "ccextractor.h"


/*----------------------------------------------------------------------
|   ReadGolomb
+---------------------------------------------------------------------*/
static unsigned int
ReadGolomb(AP4_BitStream& bits)
{
    unsigned int leading_zeros = 0;
    while (bits.ReadBit() == 0) {
        leading_zeros++;
    }
    if (leading_zeros) {
        return (1<<leading_zeros)-1+bits.ReadBits(leading_zeros);
    } else {
        return 0;
    }
}

/*----------------------------------------------------------------------
|   PrintSliceInfo
+---------------------------------------------------------------------*/
static void
PrintSliceInfo(const unsigned char* data)
{
    AP4_BitStream bits;
    bits.WriteBytes(data, 8);
    ReadGolomb(bits);
    
    unsigned int slice_type = ReadGolomb(bits);
    const char* slice_type_name = AP4_AvcParser::SliceTypeName(slice_type);
    if (slice_type_name == NULL) slice_type_name = "?";
    printf(" slice=%d (%s)", slice_type, slice_type_name);
}
#define CHECK(x) do { \
    if (!(x)) { fprintf(stderr, "ERROR line %d", __LINE__); return -1; }\
} while (0)

int avcmain(const char* filename)
{
    bool verbose = true;
   
    AP4_ByteStream* input;
    try {
        input = new AP4_FileByteStream(filename,
                               AP4_FileByteStream::STREAM_MODE_READ);
    } catch (AP4_Exception) {
        fprintf(stderr, "ERROR: cannot open input file (%s)\n", filename);
        return 1;
    }

    // get the movie
    AP4_File* file = new AP4_File(*input);
    AP4_Movie* movie = file->GetMovie();
    CHECK(movie != NULL);
    
    AP4_Track* video_track = movie->GetTrack(AP4_Track::TYPE_VIDEO);
    CHECK(video_track != NULL);
    AP4_Track* audio_track = movie->GetTrack(AP4_Track::TYPE_AUDIO);
    CHECK(video_track != NULL);
    
    AP4_LinearReader reader(*movie);
    AP4_Sample sample;
    AP4_DataBuffer sample_data;
    reader.EnableTrack(audio_track->GetId());
    reader.EnableTrack(video_track->GetId());
	    
    AP4_UI64     offset = 0;
    AP4_Cardinal audio_sample_count = 0;
    AP4_Cardinal video_sample_count = 0;
	AP4_Cardinal other_tracks_count = 0;
    do {
        AP4_UI32 track_id = 0;
        result = reader.ReadNextSample(sample, sample_data, track_id);
        if (AP4_SUCCEEDED(result)) {
            CHECK(offset < sample.GetOffset());
            offset = sample.GetOffset();
            CHECK(track_id == audio_track->GetId() || track_id == video_track->GetId());
            if (track_id == audio_track->GetId()) 
				audio_sample_count++;
			else if (track_id == video_track->GetId())
			{				
				dump ((unsigned char *) sample_data.GetData(),64,0);
				video_sample_count++;
				process_avc ((unsigned char *) sample_data.GetData(), sample_data.GetDataSize());
			}
			else
				other_tracks_count++;
            printf("track_id=%d, size=%d, offset=%lld\n", track_id, (int)sample.GetSize(), sample.GetOffset());
        } else {
            printf("track_id=%d, result=%d\n", track_id, result);
        }
    } while (AP4_SUCCEEDED(result));
    CHECK(result == AP4_ERROR_EOS);
    CHECK(reader.GetBufferFullness() == 0);
    CHECK(audio_sample_count == audio_track->GetSampleCount());
    CHECK(video_sample_count == video_track->GetSampleCount());
	printf ("Non video or audio tracks: %ld\n",(long) other_tracks_count);
    offset = 0;
    audio_sample_count = 0;
    video_sample_count = 0;
    reader.SetSampleIndex(audio_track->GetId(), 0);
    reader.SetSampleIndex(video_track->GetId(), 0);
    do {
        result = reader.ReadNextSample(audio_track->GetId(), sample, sample_data);
        if (AP4_SUCCEEDED(result)) {
            CHECK(offset < sample.GetOffset());
            offset = sample.GetOffset();
            audio_sample_count++;
            printf("size=%d, offset=%lld\n", (int)sample.GetSize(), sample.GetOffset());
        } else {
            printf("result=%d\n", result);
        }
    } while (AP4_SUCCEEDED(result));
    CHECK(result == AP4_ERROR_NOT_ENOUGH_SPACE);
    CHECK(reader.GetBufferFullness() != 0);
    
    // cleanup
    delete file;
    input->Release();

    return 0;                                           
	/*
    AP4_AvcParser parser;
    unsigned int  nalu_count = 0;
    for (;;) {
        bool eos;
        unsigned char input_buffer[40960];
        AP4_Size bytes_in_buffer = 0;
        AP4_Result result = input->ReadPartial(input_buffer, sizeof(input_buffer), bytes_in_buffer);
        if (AP4_SUCCEEDED(result)) {
            eos = false;
        } else if (result == AP4_ERROR_EOS) {
            eos = true;
        } else {
            fprintf(stderr, "ERROR: failed to read from input file\n");
            break;
        }
        AP4_Size offset = 0;
        do {
            const AP4_DataBuffer* nalu = NULL;
            AP4_Size bytes_consumed = 0;
            result = parser.Feed(&input_buffer[offset], bytes_in_buffer, bytes_consumed, nalu, eos);
            if (AP4_FAILED(result)) {
                fprintf(stderr, "ERROR: Feed() failed (%d)\n", result);
                break;
            } 
			if (nalu && nalu->GetDataSize()>0) {
                const unsigned char* nalu_payload = (const unsigned char*)nalu->GetData();
                unsigned int   nalu_type = nalu_payload[0]&0x1F;
                const char*    nalu_type_name = AP4_AvcParser::NaluTypeName(nalu_type);
                if (nalu_type_name == NULL) nalu_type_name = "UNKNOWN";
                printf("NALU %5d: size=%5d, type=%02d (%s)", 
                       nalu_count, 
                       (int)nalu->GetDataSize(),
                       nalu_type,
                       nalu_type_name);
                if (nalu_type == 9) {
                    unsigned int primary_pic_type = (nalu_payload[1]>>5);
                    const char*  primary_pic_type_name = AP4_AvcParser::PrimaryPicTypeName(primary_pic_type);
                    if (primary_pic_type_name == NULL) primary_pic_type_name = "UNKNOWN";
                    printf(" [%d:%s]\n", primary_pic_type, primary_pic_type_name);
                } else if (nalu_type == 1 || nalu_type == 2 || nalu_type == 5 || nalu_type == 19) {
                    PrintSliceInfo(&nalu_payload[1]);
                    printf("\n");
                } else {
                    printf("\n");
                }
				if (nalu_type==1 || nalu_type==5 || nalu_type==6 || nalu_type==7 || nalu_type==8 || nalu_type==9)
				process_avc((unsigned char *) nalu->GetData(),nalu->GetDataSize());
                nalu_count++;
            }
            offset += bytes_consumed;
            bytes_in_buffer -= bytes_consumed;
        } while (bytes_in_buffer);
        if (eos) break;
    }
    
    input->Release();

    return 0;
	*/
}

void mp4_loop (char *filename)
{
	printf ("Entry in MP4 loop for file: %s\n",filename);
	avcmain (filename);
}
#endif